@@ -1,14 +1,14 @@
 ;;; tree-sitter-langs.el --- Grammar bundle for tree-sitter -*- lexical-binding: t; coding: utf-8 -*-
 
-;; Copyright (C) 2021-2025 emacs-tree-sitter maintainers
+;; Copyright (C) 2021-2026 emacs-tree-sitter maintainers
 ;;
 ;; Author: Tuấn-Anh Nguyễn <ubolonton@gmail.com>
 ;; Maintainer: Jen-Chieh Shen <jcs090218@gmail.com>
 ;; Keywords: languages tools parsers tree-sitter
 ;; Homepage: https://github.com/emacs-tree-sitter/tree-sitter-langs
-;; Package-Version: 0.12.319
-;; Package-Revision: 74a1ab3f9970
-;; Package-Requires: ((emacs "25.1") (tree-sitter "0.15.0"))
+;; Package-Version: 0.13.20
+;; Package-Revision: 758dcce63b2c
+;; Package-Requires: ((emacs "26.1") (tree-sitter "0.15.0"))
 ;; SPDX-License-Identifier: MIT
 
 ;;; Commentary:
@@ -96,7 +96,7 @@ See `tree-sitter-langs-repos'."
 (advice-add 'tree-sitter-load :before #'tree-sitter-langs--init-load-path)
 
 ;;;###autoload
-(defun tree-sitter-langs--init-major-mode-alist (&rest _args)
+(defun tree-sitter-langs--init-major-mode-table (&rest _args)
   "Link known major modes to languages provided by the bundle."
   (let ((alist '((actionscript-mode      . actionscript)
                  (ada-mode               . ada)
@@ -115,14 +115,17 @@ See `tree-sitter-langs-repos'."
                  (bibtex-mode            . bibtex)
                  (c-mode                 . c)
                  (caml-mode              . ocaml)
+                 (cg-mode                . cg)
                  (clojure-mode           . clojure)
                  (lisp-mode              . commonlisp)
                  (lisp-interaction-mode  . commonlisp)
                  (csharp-mode            . c-sharp)
                  (c++-mode               . cpp)
                  (cmake-mode             . cmake)
+                 (crystal-mode           . crystal)
                  (d-mode                 . d)
                  (dart-mode              . dart)
+                 (diff-mode              . diff)
                  (dockerfile-mode        . dockerfile)
                  (css-mode               . css)
                  (csv-mode               . csv)
@@ -135,10 +138,13 @@ See `tree-sitter-langs-repos'."
                  (fennel-mode            . fennel)
                  (f90-mode               . fortran)
                  (fortran-mode           . fortran)
+                 (fsharp-mode            . fsharp)
                  (gdscript-mode          . gdscript)
+                 (gdshader-mode          . gdshader)
                  (git-commit-mode        . gitcommit)
                  (git-rebase-mode        . git-rebase)
                  (gitattributes-mode     . gitattributes)
+                 (gitconfig-mode         . git-config)
                  (gitignore-mode         . gitignore)
                  (gleam-mode             . gleam)
                  (glsl-mode              . glsl)
@@ -150,6 +156,7 @@ See `tree-sitter-langs-repos'."
                  (haskell-mode           . haskell)
                  (haxe-mode              . haxe)
                  (hcl-mode               . hcl)
+                 (hfst-mode              . lexc)
                  (terraform-mode         . hcl)
                  (heex-mode              . heex)
                  (hlsl-mode              . hlsl)
@@ -184,6 +191,7 @@ See `tree-sitter-langs-repos'."
                  (matlab-mode            . matlab)
                  (mermaid-mode           . mermaid)
                  (meson-mode             . meson)
+                 (nim-mode               . nim)
                  (ninja-mode             . ninja)
                  (noir-mode              . noir)
                  (ocaml-mode             . ocaml)
@@ -199,7 +207,9 @@ See `tree-sitter-langs-repos'."
                  (pygn-mode              . pgn)
                  (racket-mode            . racket)
                  (rjsx-mode              . javascript)
+                 (ron-mode               . ron)
                  (rst-mode               . rst)
+                 (rtx-mode               . rtx)
                  (ruby-mode              . ruby)
                  (rust-mode              . rust)
                  (rustic-mode            . rust)
@@ -221,33 +231,29 @@ See `tree-sitter-langs-repos'."
                  (typst-mode             . typst)
                  (verilog-mode           . verilog)
                  (vhdl-mode              . vhdl)
+                 (vimrc-mode             . vim)
+                 (wat-mode               . wat)
+                 (wgsl-mode              . wgsl)
                  (nxml-mode              . xml)
                  (yaml-mode              . yaml)
                  (k8s-mode               . yaml)
                  (zig-mode               . zig))))
-    (cond
-     ;; If never been set; update the value directly!
-     ((null tree-sitter-major-mode-language-alist)
-      (setq tree-sitter-major-mode-language-alist alist))
-     ;; Else, we try not to change the custom value from user.
-     (t
-      (dolist (entry alist)
-        ;; TODO: The `cl-pushnew' operation is very slow; better if we can use
-        ;; `hash-table' instead?
-        ;;
-        ;; I've run the benchmark tests with 123 entries (alist length);
-        ;; compare to set value directly. Obviously, set the value directly cost
-        ;; constant time but how fast compare to `cl-pushnew'?
-        ;;
-        ;; It's average 325 to 650 times faster! The average time for each
-        ;; `cl-pushnew' operation is 2.6 to 5.2 times slower.
-        (cl-pushnew entry tree-sitter-major-mode-language-alist :key #'car))))
-    (advice-remove 'tree-sitter--setup #'tree-sitter-langs--init-major-mode-alist)))
+    ;; Create one.
+    (when (null tree-sitter-major-mode-language-table)
+      (setq tree-sitter-major-mode-language-table (make-hash-table :test 'eq)))
+    ;; Add entry from the alist.
+    (dolist (entry alist)
+      (when-let* ((key (car entry))
+                  (value (cdr entry)))
+        (unless (gethash key tree-sitter-major-mode-language-table)
+          (puthash key value tree-sitter-major-mode-language-table))))
+    ;; Clean up.
+    (advice-remove 'tree-sitter--setup #'tree-sitter-langs--init-major-mode-table)))
 
 ;;;###autoload
-(advice-add 'tree-sitter--setup :before #'tree-sitter-langs--init-major-mode-alist)
+(advice-add 'tree-sitter--setup :before #'tree-sitter-langs--init-major-mode-table)
 ;;; Normal load.
-(tree-sitter-langs--init-major-mode-alist)
+(tree-sitter-langs--init-major-mode-table)
 
 (defun tree-sitter-langs--hl-query-path (lang-symbol &optional mode)
   "Return the highlighting query file for LANG-SYMBOL.
